@@ -15,7 +15,7 @@ our @ISA = qw(DynaLoader);
 # SQL_DRIVER_VER is formatted as dd.dd.dddd
 # for version 5.x please switch to 5.00(_00) version numbering
 # keep $VERSION in Bundle/DBD/mysql.pm in sync
-our $VERSION = '4.032_01';
+our $VERSION = '4.032_03';
 
 bootstrap DBD::mysql $VERSION;
 
@@ -129,6 +129,8 @@ sub connect {
     $username ||= '';
     $password ||= '';
     $attrhash ||= {};
+    $attrhash->{mysql_conn_attrs} ||= {};
+    $attrhash->{mysql_conn_attrs}->{'program_name'} ||= $0;
 
     # create a 'blank' dbh
     my($this, $privateAttrHash) = (undef, $attrhash);
@@ -1256,6 +1258,87 @@ Example:
 
 $testdsn="DBI:mysqlEmb:database=test;mysql_embedded_groups=embedded_server,common";
 
+=item mysql_conn_attrs
+
+The option <mysql_conn_attrs> is a hash of attribute names and values which can be
+used to send custom connection attributes to the server. Some attributes like
+'_os', '_platform', '_client_name' and '_client_version' are added by libmysqlclient
+and 'program_name' is added by DBD::mysql.
+
+You can then later read these attributes from the performance schema tables which
+can be quite helpful for profiling your database or creating statistics.
+You'll have to use a MySQL 5.6 server and libmysqlclient or newer to leverage this
+feature.
+
+  my $dbh= DBI->connect($dsn, $user, $password,
+    { AutoCommit => 0,
+      mysql_conn_attrs => {
+        foo => 'bar',
+        wiz => 'bang'
+      },
+    });
+
+Now you can select the results from the performance schema tables. You can do this
+in the same session, but also afterwards. It can be very useful to answer questions
+like 'which script sent this query?'.
+
+  my $results = $dbh->selectall_hashref(
+    'SELECT * FROM performance_schema.session_connect_attrs',
+    'ATTR_NAME'
+  );
+
+This returns:
+
+  $result = {
+    'foo' => {
+        'ATTR_VALUE'       => 'bar',
+        'PROCESSLIST_ID'   => '3',
+        'ATTR_NAME'        => 'foo',
+        'ORDINAL_POSITION' => '6'
+    },
+    'wiz' => {
+        'ATTR_VALUE'       => 'bang',
+        'PROCESSLIST_ID'   => '3',
+        'ATTR_NAME'        => 'wiz',
+        'ORDINAL_POSITION' => '3'
+    },
+    'program_name' => {
+        'ATTR_VALUE'       => './foo.pl',
+        'PROCESSLIST_ID'   => '3',
+        'ATTR_NAME'        => 'program_name',
+        'ORDINAL_POSITION' => '5'
+    },
+    '_client_name' => {
+        'ATTR_VALUE'       => 'libmysql',
+        'PROCESSLIST_ID'   => '3',
+        'ATTR_NAME'        => '_client_name',
+        'ORDINAL_POSITION' => '1'
+    },
+    '_client_version' => {
+        'ATTR_VALUE'       => '5.6.24',
+        'PROCESSLIST_ID'   => '3',
+        'ATTR_NAME'        => '_client_version',
+        'ORDINAL_POSITION' => '7'
+    },
+    '_os' => {
+        'ATTR_VALUE'       => 'osx10.8',
+        'PROCESSLIST_ID'   => '3',
+        'ATTR_NAME'        => '_os',
+        'ORDINAL_POSITION' => '0'
+    },
+    '_pid' => {
+        'ATTR_VALUE'       => '59860',
+        'PROCESSLIST_ID'   => '3',
+        'ATTR_NAME'        => '_pid',
+        'ORDINAL_POSITION' => '2'
+    },
+    '_platform' => {
+        'ATTR_VALUE'       => 'x86_64',
+        'PROCESSLIST_ID'   => '3',
+        'ATTR_NAME'        => '_platform',
+        'ORDINAL_POSITION' => '4'
+    }
+  };
 
 =back
 
@@ -1481,6 +1564,11 @@ need to issue the command C<SET NAMES utf8> to get the same effect.
 
 This option is experimental and may change in future versions.
 
+=item mysql_enable_utf8mb4
+
+This is similar to mysql_enable_utf8, but is capable of handling 4-byte
+UTF-8 characters.
+
 =item mysql_bind_type_guessing
 
 This attribute causes the driver (emulated prepare statements)
@@ -1498,34 +1586,34 @@ character column, if this column is indexed, if you query that
 column with the integer value not being quoted, it will not
 use the index:
 
-MariaDB [test]> explain select * from test where value0 = '3' \G
-*************************** 1. row ***************************
-           id: 1
-  select_type: SIMPLE
-        table: test
-         type: ref
-possible_keys: value0
-          key: value0
-      key_len: 13
-          ref: const
-         rows: 1
-        Extra: Using index condition
-1 row in set (0.00 sec)
+    MariaDB [test]> explain select * from test where value0 = '3' \G
+    *************************** 1. row ***************************
+               id: 1
+      select_type: SIMPLE
+            table: test
+             type: ref
+    possible_keys: value0
+              key: value0
+          key_len: 13
+              ref: const
+             rows: 1
+            Extra: Using index condition
+    1 row in set (0.00 sec)
 
-MariaDB [test]> explain select * from test where value0 = 3
-    -> \G
-*************************** 1. row ***************************
-           id: 1
-  select_type: SIMPLE
-        table: test
-         type: ALL
-possible_keys: value0
-          key: NULL
-      key_len: NULL
-          ref: NULL
-         rows: 6
-        Extra: Using where
-1 row in set (0.00 sec)
+    MariaDB [test]> explain select * from test where value0 = 3
+        -> \G
+    *************************** 1. row ***************************
+               id: 1
+      select_type: SIMPLE
+            table: test
+             type: ALL
+    possible_keys: value0
+              key: NULL
+          key_len: NULL
+              ref: NULL
+             rows: 6
+            Extra: Using where
+    1 row in set (0.00 sec)
 
 See bug: https://rt.cpan.org/Ticket/Display.html?id=43822
 
@@ -1751,7 +1839,7 @@ The methods
     $dbh->rollback();
     $dbh->commit();
 
-will issue the commands COMMIT and ROLLBACK, respectively. A
+will issue the commands ROLLBACK and COMMIT, respectively. A
 ROLLBACK will also be issued if AutoCommit mode is off and the
 database handles DESTROY method is called. Again, this is following
 the DBI specifications.
@@ -1864,7 +1952,7 @@ DBD::mysql code snippets will be added in the future.
 =head2 Issues with multiple result sets
 
 Please be aware ther could be issues if your result sets are "jagged",
-meaning the number of columns of your results vary. Varying numbers of 
+meaning the number of columns of your results vary. Varying numbers of
 columns could result in your script crashing.
 
 
@@ -1938,7 +2026,7 @@ support).
 For the past nine years DBD::mysql has been maintained by
 Patrick Galbraith (I<patg@patg.net>), and recently with the great help of
 Michiel Beijen (I<michiel.beijen@gmail.com>),  along with the entire community
-of Perl developers who keep sending patches to help continue improving DBD::mysql 
+of Perl developers who keep sending patches to help continue improving DBD::mysql
 
 
 =head1 CONTRIBUTIONS
